@@ -45,10 +45,12 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
     'ecephys_676909_2023-12-13_13-43-40_sorted_2023-12-17_03-16-51'
 
     Access subsets of data assets by sorter name:
-    >>> session.ecephys.sorter.kilosort2_5.data_assets[0].id
+    >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets[0].id
     '1e11bdf5-b452-4fd9-bbb1-48383a9b0842'
-    >>> session.ecephys.sorter.kilosort2_5.data_assets[0].name
+    >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets[0].name
     'ecephys_676909_2023-12-13_13-43-40_sorted_2023-12-17_03-16-51'
+    >>> session.ecephys.sorter.names
+    ('kilosort2_5',)
 
     Returned models are enhanced with sorting pipeline-related properties:
     >>> session.ecephys.sorted_data_assets[0].sorted_probes
@@ -57,6 +59,8 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
     'kilosort2_5'
     """
 
+    _base: aind_session.Session
+
     DEFAULT_SORTING_PIPELINE_ID: ClassVar[str] = "1f8f159a-7670-47a9-baf1-078905fc9c2e"
     DEFAULT_TRIGGER_CAPSULE_ID: ClassVar[str] = "eb5a26e4-a391-4d79-9da5-1ab65b71253f"
 
@@ -76,11 +80,11 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         """
         if (
             path := EcephysExtension.get_clipped_and_compressed_dirs(
-                self._session.raw_data_asset.id
+                self._base.raw_data_asset.id
             )[0]
         ) is None:
             raise AttributeError(
-                f"No 'clipped' dir found in uploaded raw data for {self._session.id} (checked in root dir and modality subdirectory)"
+                f"No 'clipped' dir found in uploaded raw data for {self._base.id} (checked in root dir and modality subdirectory)"
             )
         return path
 
@@ -101,11 +105,11 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         """
         if (
             path := EcephysExtension.get_clipped_and_compressed_dirs(
-                self._session.raw_data_asset.id
+                self._base.raw_data_asset.id
             )[1]
         ) is None:
             raise AttributeError(
-                f"No 'compressed' dir found in uploaded raw data for {self._session.id} (checked in root dir and modality subdirectory)"
+                f"No 'compressed' dir found in uploaded raw data for {self._base.id} (checked in root dir and modality subdirectory)"
             )
         return path
 
@@ -164,7 +168,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         if self.sorted_data_assets[-1].is_sorting_error:
             return False
         logger.debug(
-            f"The latest sorted data asset for {self._session.id} appears to have been sorted successfully: {self.sorted_data_assets[-1].id}"
+            f"The latest sorted data asset for {self._base.id} appears to have been sorted successfully: {self.sorted_data_assets[-1].id}"
         )
         return True
 
@@ -189,11 +193,11 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         """
         assets = tuple(
             EcephysExtension.get_sorted_data_asset_model(asset)
-            for asset in self._session.data_assets
-            if EcephysExtension.is_sorted_data_asset(asset)
+            for asset in self._base.data_assets
+            if EcephysExtension.is_sorted_data_asset(asset.id)
         )
         logger.debug(
-            f"Found {len(assets)} sorted data asset{'' if len(assets) == 1 else 's'} for {self._session.id}"
+            f"Found {len(assets)} sorted data asset{'' if len(assets) == 1 else 's'} for {self._base.id}"
         )
         return assets
 
@@ -272,6 +276,13 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
             """
             return aind_session.utils.codeocean_utils.is_output_error(self.output)
 
+        @property
+        def is_sorting_analyzer(self) -> bool:
+            """The sorting pipeline used the `SortingAnalyzer` introduced in
+            `SpikeInterface==0.101.1`. Results are organized in `.zarr` format.
+            """
+            return EcephysExtension.is_sorting_analyzer_asset(self.id)
+
     @staticmethod
     def get_sorted_data_asset_model(
         asset_id: str | codeocean.data_asset.DataAsset,
@@ -291,7 +302,8 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         )
 
     @staticmethod
-    def is_sorted_data_asset(asset_id: str | codeocean.data_asset.DataAsset) -> bool:
+    @functools.cache
+    def is_sorted_data_asset(asset_id: str) -> bool:
         """Check if the asset is a sorted data asset.
 
         - assumes sorted asset to be named `<session-id>_sorted<unknown-suffix>`
@@ -324,6 +336,30 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
             )
             return False
 
+    @staticmethod
+    @functools.cache
+    def is_sorting_analyzer_asset(
+        asset_id: str,
+    ) -> bool:
+        """The sorting pipeline used the `SortingAnalyzer` introduced in
+        `SpikeInterface==0.101.1`.
+
+        - checks if results are organized in `.zarr` format
+        #TODO use spikeinterface version instead
+
+        Examples
+        --------
+
+        >>> aind_session.ecephys.is_sorting_analyzer_asset('616375f1-836b-4187-823f-3457f07b7223')
+        True
+        >>> aind_session.ecephys.is_sorting_analyzer_asset('1e11bdf5-b452-4fd9-bbb1-48383a9b0842')
+        False
+        """
+        asset = EcephysExtension.get_sorted_data_asset_model(asset_id)
+        if next(asset.path.glob("*/*.zarr"), None):
+            return True
+        return False
+
     @property
     def sorter(self) -> _SorterNamespace:
         """Namespace for accessing sorting pipeline output for different sorter
@@ -337,7 +373,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         >>> session.ecephys.sorter.kilosort4
         SorterExtension(Session('ecephys_676909_2023-12-13_13-43-40'))
         """
-        return EcephysExtension._SorterNamespace(parent=self)
+        return EcephysExtension._SorterNamespace(base=self)
 
     class _SorterNamespace:
         """Namespace for accessing sorting pipeline output for different
@@ -349,17 +385,19 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
           session's sorted data assets
         """
 
+        _base: EcephysExtension
+
         # known sorter names can be added here to aid static typing/autocomplete:
         kilosort2_5: EcephysExtension.SorterExtension
         kilosort4: EcephysExtension.SorterExtension
         spykingcircus2: EcephysExtension.SorterExtension
 
-        def __init__(self, parent: EcephysExtension):
-            self._parent = parent
+        def __init__(self, base: EcephysExtension):
+            self._base = base
 
         def __getattr__(self, sorter_name: str) -> EcephysExtension.SorterExtension:
             return EcephysExtension.SorterExtension(
-                ecephys=self._parent, sorter_name=sorter_name
+                ecephys=self._base, sorter_name=sorter_name
             )
 
         @property
@@ -382,7 +420,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
                         set(
                             executor.map(
                                 EcephysExtension.get_sorter_name,
-                                self._parent.sorted_data_assets,
+                                (asset.id for asset in self._base.sorted_data_assets),
                             )
                         )
                     )
@@ -394,28 +432,28 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         assets created by a specific sorter"""
 
         def __init__(self, ecephys: EcephysExtension, sorter_name: str) -> None:
-            super().__init__(session=ecephys._session)
+            super().__init__(base=ecephys._base)
             self._ecephys = ecephys
             self._sorter_name = sorter_name
 
         @property
-        def data_assets(self) -> tuple[EcephysExtension.SortedDataAsset, ...]:
+        def sorted_data_assets(self) -> tuple[EcephysExtension.SortedDataAsset, ...]:
             """All data assets produced using the given SpikeInterface `sorter_name`
             associated with the session (may be empty).
 
             Examples
             --------
             >>> session = aind_session.Session('ecephys_676909_2023-12-13_13-43-40')
-            >>> session.ecephys.sorted_data_assets[0].id
+            >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets[0].id
             '1e11bdf5-b452-4fd9-bbb1-48383a9b0842'
-            >>> session.ecephys.sorted_data_assets[0].name
+            >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets[0].name
             'ecephys_676909_2023-12-13_13-43-40_sorted_2023-12-17_03-16-51'
-            >>> session.ecephys.sorted_data_assets[0].created
+            >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets[0].created
             1702783011
 
             Empty if no sorted data assets are found:
             >>> session = aind_session.Session('ecephys_676909_2023-12-13_13-43-39')
-            >>> session.ecephys.sorted_data_assets
+            >>> session.ecephys.sorter.kilosort2_5.sorted_data_assets
             ()
             """
             with concurrent.futures.ThreadPoolExecutor() as executor:
@@ -434,7 +472,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
                 if sorter_name == self._sorter_name:
                     assets.append(asset)
             logger.debug(
-                f"Found {len(assets)} {self._sorter_name} sorted data asset{'' if len(assets) == 1 else 's'} for {self._session.id}"
+                f"Found {len(assets)} {self._sorter_name} sorted data asset{'' if len(assets) == 1 else 's'} for {self._base.id}"
             )
             return tuple(assets)
 
@@ -511,16 +549,16 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
           currently awkward: will update to pass named parameter kwargs in the
           future
             - if needed, you can override the parameters used with a custom list
-        - if `skip_already_sorting` is `True`, a new pipeline run will not
-          triggered the session's raw data asset is already being sorted (returns
-          None)
+        - if `skip_already_sorting` is `True`, a new pipeline run will not be
+          triggered if the session's raw data asset is already being sorted
+          (and this function returns None)
 
         Examples
         --------
         >>> session = aind_session.Session('ecephys_676909_2023-12-13_13-43-40')
         >>> computation = session.ecephys.run_sorting()       # doctest: +SKIP
 
-        # Supply list of positional arguments (pipeline type and data asset ID are
+        Supply list of positional arguments (pipeline type and data asset ID are
         required)
         >>> override_parameters = ['ecephys_opto', session.raw_data_asset.id]
         >>> session.ecephys.run_sorting(override_parameters) # doctest: +SKIP
@@ -536,7 +574,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
                 parameters[1]
             )
         else:
-            asset = self._session.raw_data_asset
+            asset = self._base.raw_data_asset
             parameters = [pipeline_type, asset.id]
         if skip_already_sorting:
             current_computations = (
@@ -588,7 +626,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         """
         return EcephysExtension.get_current_sorting_pipeline_computations(
             pipeline_id=self.DEFAULT_SORTING_PIPELINE_ID,
-            raw_data_asset_id_or_model=self._session.raw_data_asset.id,
+            raw_data_asset_id_or_model=self._base.raw_data_asset.id,
         )
 
     @staticmethod
@@ -642,7 +680,7 @@ class EcephysExtension(aind_session.extension.ExtensionBaseClass):
         --------
 
         - processing.json['processing_pipeline']['data_processes'][index]['parameters']['sorter_name']:
-        >>> aind_session.ecephys.get_sorter_name('921a186a-d8ff-4efc-8e1a-891fde8cd394')
+        >>> aind_session.ecephys.get_sorter_name('d50a3447-7f12-4da7-83c5-845744c4d4f9')
         'kilosort2_5'
 
         - processing.json['processing_pipeline']['data_processes'][index]['parameters']['sorter_name']:
