@@ -54,9 +54,9 @@ class Session:
     ('behavior', 'behavior_videos', 'ecephys')
 
     Should be able to handle all platforms:
-    >>> session = Session('multiplane-ophys_741863_2024-08-13_09-26-41')
+    >>> session = Session('multiplane-ophys_721291_2024-05-08_08-05-54')
     >>> session.raw_data_dir.as_posix()
-    's3://aind-private-data-prod-o5171v/multiplane-ophys_741863_2024-08-13_09-26-41'
+    's3://aind-private-data-prod-o5171v/multiplane-ophys_721291_2024-05-08_08-05-54'
 
     >>> session = Session('behavior_717121_2024-06-16_11-39-34')
     >>> session.raw_data_dir.as_posix()
@@ -74,6 +74,14 @@ class Session:
 
     """
 
+    id: str
+    subject_id: str
+    platform: str
+    dt: datetime.datetime
+    date: npc_session.DateRecord
+    time: npc_session.TimeRecord
+    datetime: npc_session.DatetimeRecord
+
     # optional annotations for extensions here to enable IDE type checking,
     # autocompletion, etc.
     ecephys: aind_session.extensions.ecephys.EcephysExtension  # type: ignore [name-defined]
@@ -153,9 +161,11 @@ class Session:
         >>> session = aind_session.Session('ecephys_676909_2023-12-13_13-43-40')
         >>> session.data_assets[0].name
         'ecephys_676909_2023-12-13_13-43-40'
+        >>> session = aind_session.Session('SmartSPIM_738819_2024-06-21_13-48-58')
+        >>> assert session.data_assets
         """
-        return aind_session.utils.get_session_data_assets(
-            session_id=self.id,
+        return aind_session.utils.get_data_assets(
+            self.id,
             ttl_hash=aind_session.utils.get_ttl_hash(),
         )
 
@@ -306,6 +316,9 @@ class Session:
         >>> session = aind_session.Session('ecephys_676909_2023-12-13_13-43-40')
         >>> session.modalities
         ('behavior', 'behavior_videos', 'ecephys')
+        >>> session = aind_session.Session('behavior_676909_2023-10-24_15-15-50')
+        >>> session.modalities
+        ('behavior',)
         """
         if not self.is_uploaded:
             logger.info(
@@ -316,11 +329,12 @@ class Session:
             d.name for d in self.raw_data_dir.iterdir() if d.is_dir()
         }
         for name in ("ecephys_compressed", "ecephys_clipped"):
-            dir_names.remove(name)
-            logger.debug(
-                f"Returning modality names with {name!r} represented as 'ecephys'"
-            )
-            dir_names.add("ecephys")
+            if name in tuple(dir_names):
+                dir_names.remove(name)
+                logger.debug(
+                    f"Returning modality names with {name!r} represented as 'ecephys'"
+                )
+                dir_names.add("ecephys")
         for term in ("metadata",):
             for name in tuple(dir_names):
                 if term in name:
@@ -447,7 +461,7 @@ def get_sessions(
         sessions.add(session)
     if not sessions:
         logger.info(f"No sessions found matching {parameters=}")
-    return tuple(sorted(sessions))
+    return tuple(sorted(sessions, key=lambda s: s.dt))
 
 
 if __name__ == "__main__":
