@@ -3,13 +3,14 @@
 from __future__ import annotations
 
 import contextlib
+import datetime
 import functools
 import logging
 import os
 import time
 import uuid
 from collections.abc import Iterable
-from typing import Any, Literal
+from typing import Any, Literal, overload
 
 import codeocean
 import codeocean.components
@@ -18,14 +19,41 @@ import codeocean.data_asset
 import npc_session
 import requests  # type: ignore # to avoid checking types/installing types-requests
 import upath
+import urllib3
+
+try:
+    from codeocean.error import Error as CodeOceanError
+except ImportError:
+
+    class CodeOceanError(requests.HTTPError):
+        pass
+
 
 import aind_session.utils
+import aind_session.utils.docdb_utils
 
 logger = logging.getLogger(__name__)
 
+DEFAULT_CO_RETRY = urllib3.Retry(
+    total=5,
+    backoff_factor=0.5,
+    status_forcelist=[429, 500, 502, 503, 504],
+    allowed_methods=["HEAD", "GET", "PUT", "POST", "DELETE", "OPTIONS", "TRACE"],
+)
+
+
+def _get_status(exc: CodeOceanError | requests.HTTPError) -> int:
+    if isinstance(exc, requests.HTTPError):
+        return exc.response.status_code
+    else:  # the real exception from codeocean sdk is not a subclass of requests.HTTPError
+        assert isinstance(exc, CodeOceanError)
+        return exc.status_code
+
 
 @functools.cache
-def get_codeocean_client(check_credentials: bool = True) -> codeocean.CodeOcean:
+def get_codeocean_client(
+    check_credentials: bool = True, retries: int | urllib3.Retry = DEFAULT_CO_RETRY
+) -> codeocean.CodeOcean:
     """
     Get a CodeOcean client using environment variables.
 
@@ -52,12 +80,14 @@ def get_codeocean_client(check_credentials: bool = True) -> codeocean.CodeOcean:
         raise KeyError(
             "`CODE_OCEAN_API_TOKEN` not found in environment variables and no `COP_` variable found"
         )
+    domain = os.getenv(
+        key="CODE_OCEAN_DOMAIN",
+        default="https://codeocean.allenneuraldynamics.org",
+    )
     client = codeocean.CodeOcean(
-        domain=os.getenv(
-            key="CODE_OCEAN_DOMAIN",
-            default="https://codeocean.allenneuraldynamics.org",
-        ),
+        domain=domain,
         token=token,
+        retries=retries,
     )
     if check_credentials:
         logger.debug(
@@ -74,8 +104,9 @@ def get_codeocean_client(check_credentials: bool = True) -> codeocean.CodeOcean:
                     favorite=False,
                 )
             )
-        except requests.HTTPError as exc:
-            if exc.response.status_code == 401:
+        except (requests.HTTPError, CodeOceanError) as exc:
+            status_code = _get_status(exc)
+            if status_code == 401:
                 raise ValueError(
                     "CodeOcean API token was found in environment variables, but does not have permissions to read datasets: check `CODE_OCEAN_API_TOKEN`"
                 ) from None
@@ -175,8 +206,9 @@ def get_codeocean_model(
         return get_codeocean_client().computations.get_computation(
             get_normalized_uuid(asset_or_computation_id)
         )
-    except requests.HTTPError as exc:
-        if exc.response.status_code == 404:
+    except (requests.HTTPError, CodeOceanError) as exc:
+        status_code = _get_status(exc)
+        if status_code == 404:
             raise ValueError(
                 f"No data asset or computation found matching ID {asset_or_computation_id}"
             ) from None
@@ -205,8 +237,9 @@ def get_data_asset_model(
         return get_codeocean_client().data_assets.get_data_asset(
             get_normalized_uuid(asset_id_or_model)
         )
-    except requests.HTTPError as exc:
-        if exc.response.status_code == 404:
+    except (requests.HTTPError, CodeOceanError) as exc:
+        status_code = _get_status(exc)
+        if status_code == 404:
             raise ValueError(
                 f"No data asset found matching ID {asset_id_or_model}"
             ) from exc
@@ -223,12 +256,20 @@ def sort_by_created(
     ],
 ) -> tuple[codeocean.data_asset.DataAsset, ...]:
     """Sort data assets or computations by ascending creation date. Accepts IDs or models."""
-    return tuple(
-        sorted(
-            (get_codeocean_model(a) for a in ids_or_models),
-            key=lambda asset: asset.created,
-        )
-    )
+    models = []
+    for id_or_model in ids_or_models:
+        try:
+            model = get_codeocean_model(id_or_model)
+        except (requests.HTTPError, CodeOceanError) as exc:
+            status_code = _get_status(exc)
+            if status_code in (401, 404):
+                logger.info(
+                    f"Skipping {id_or_model} as it is not accessible with current credentials"
+                )
+                continue
+            raise
+        models.append(model)
+    return tuple(sorted(models, key=lambda asset: asset.created))
 
 
 @functools.cache
@@ -302,6 +343,51 @@ def get_data_asset_source_dir(
     return get_dir_from_known_s3_locations(asset)
 
 
+def wait_until_ready(
+    data_asset: str | codeocean.data_asset.DataAsset,
+    check_files: bool = True,
+    timeout: float | None = 60,
+    **kwargs,
+) -> codeocean.data_asset.DataAsset:
+    """A wrapper around `codeocean.data_asset.DataAssets.wait_until_ready` that optionally waits for files to appear in the asset's source dir.
+
+    When a new data asset is created, it may take some time for the files to be visible in the 'source_bucket' specified in the asset's metadata.
+    This function will wait for the asset to be 'Ready' and, if `check_files=True`, will also wait for at least one file to appear in the source directory.
+
+    - extra kwargs are passed to `codeocean.data_asset.DataAssets.wait_until_ready`
+    - `timeout` is in seconds
+
+    Examples
+    --------
+    >>> updated_asset = wait_until_ready('16d46411-540a-4122-b47f-8cb2a15d593a', check_files=True)
+    """
+    asset = get_data_asset_model(data_asset)
+    t0 = time.time()
+    logger.debug(f"Waiting for asset {asset.name} to be ready")
+    updated_asset = get_codeocean_client().data_assets.wait_until_ready(
+        data_asset=asset,
+        timeout=timeout,
+        **kwargs,
+    )
+    polling_interval = 1
+    if check_files:
+        source_dir = get_data_asset_source_dir(updated_asset.id)
+        logger.debug(f"Waiting for files to appear in {source_dir}")
+        while timeout is None or time.time() - t0 < timeout:
+            if source_dir.exists() and any(source_dir.glob("*")):
+                break
+            logger.debug(
+                f"No files found in {source_dir}: waiting {polling_interval} s before checking again"
+            )
+            time.sleep(polling_interval)
+        else:
+            raise TimeoutError(
+                f"Failed to find any files in {source_dir} after {timeout} s"
+            )
+    logger.debug(f"Asset {asset.name} is ready: returning updated asset")
+    return get_data_asset_model(updated_asset.id)
+
+
 def get_data_asset_search_query(
     name: str | None = None,
     subject_id: str | int | None = None,
@@ -315,6 +401,9 @@ def get_data_asset_search_query(
     Note: current understanding of the operation of the 'query' field is largely
     undocumented, so the following is based on empirical testing and is not
     exhaustive.
+
+    - if `name` is over 20 characters long, no results are returned: we will truncate it in this function and raise a
+      warning
     """
     params = {
         k: v for k, v in locals().items() if v not in ("", None)
@@ -330,6 +419,12 @@ def get_data_asset_search_query(
             tags = v if (isinstance(v, Iterable) and not isinstance(v, str)) else (v,)
             for t in tags:
                 append(k, t)
+        if k == "name" and len(v) > 20:
+            v = v[:20]
+            logger.warning(
+                f"Data asset search in CodeOcean with `name` over 20 characters long is broken: truncating to {v!r}"
+            )
+            append(k, v)
         else:
             append(k, v)
     query_text = " ".join(query)
@@ -337,12 +432,29 @@ def get_data_asset_search_query(
     return query_text
 
 
+@overload
 def search_data_assets(
     search_params: dict[str, Any] | codeocean.data_asset.DataAssetSearchParams,
+    as_dict: Literal[False] = False,
     page_size: int = 100,
     max_pages: int = 1000,
     raise_on_page_limit: bool = True,
-) -> tuple[codeocean.data_asset.DataAsset, ...]:
+) -> tuple[codeocean.data_asset.DataAsset, ...]: ...
+@overload
+def search_data_assets(
+    search_params: dict[str, Any] | codeocean.data_asset.DataAssetSearchParams,
+    as_dict: Literal[True] = True,
+    page_size: int = 100,
+    max_pages: int = 1000,
+    raise_on_page_limit: bool = True,
+) -> tuple[dict[str, Any], ...]: ...
+def search_data_assets(
+    search_params: dict[str, Any] | codeocean.data_asset.DataAssetSearchParams,
+    as_dict: bool = False,
+    page_size: int = 100,
+    max_pages: int = 1000,
+    raise_on_page_limit: bool = True,
+) -> tuple[codeocean.data_asset.DataAsset | dict[str, Any], ...]:
     """A wrapper around `codeocean.data_assets.search_data_assets` that makes it
     slightly easier to use.
 
@@ -352,8 +464,9 @@ def search_data_assets(
     - fills in required fields with sensible defaults if not provided:
         - `archived=False`
         - `favorite=False`
-    - raises a `ValueError` if the page limit is reached, unless
-      `raise_on_page_limit=False`
+    - raises a `ValueError` if the page limit is reached, unless `raise_on_page_limit=False`
+    - `as_dict=True` returns json results without converting to dataclasses
+        - this will be much faster for large numbers of results
 
     Examples
     --------
@@ -384,19 +497,31 @@ def search_data_assets(
         f"Fetching data assets results matching search parameters: {updated_params}"
     )
 
-    assets: list[codeocean.data_asset.DataAsset] = []
+    assets: list[codeocean.data_asset.DataAsset | dict[str, Any]] = []
     page = 0
     while page < max_pages:
-        search_results = get_codeocean_client().data_assets.search_data_assets(
-            codeocean.data_asset.DataAssetSearchParams(
-                limit=page_size,
-                offset=page * page_size,
-                **updated_params,
-            )
+        search_params = codeocean.data_asset.DataAssetSearchParams(
+            limit=page_size,
+            offset=page * page_size,
+            **updated_params,
         )
-        assets.extend(search_results.results)
-        if not search_results.has_more:
-            break
+        if not as_dict:
+            search_results = get_codeocean_client().data_assets.search_data_assets(
+                search_params
+            )
+            assets.extend(search_results.results)
+            if not search_results.has_more:
+                break
+        else:
+            search_results = (
+                get_codeocean_client()
+                .session.post("data_assets/search", json=search_params.to_dict())
+                .json()
+            )
+            # requests session already has `raise_for_status` hook
+            assets.extend(search_results["results"])
+            if not search_results["has_more"]:
+                break
         page += 1
     else:
         if raise_on_page_limit:
@@ -419,6 +544,9 @@ def search_computations(
     in_progress: bool | None = None,
     computation_state: codeocean.computation.ComputationState | None = None,
     ttl_hash: int | None = None,
+    date: str | datetime.date | datetime.datetime | None = None,
+    start_date: str | datetime.date | datetime.datetime | None = None,
+    end_date: str | datetime.date | datetime.datetime | None = None,
 ) -> tuple[codeocean.computation.Computation, ...]:
     """
     Search for capsule or pipeline computations with specific attributes.
@@ -456,6 +584,10 @@ def search_computations(
     >>> computations = search_computations(pipeline_id, attached_data_asset_id="83636983-f80d-42d6-a075-09b60c6abd5e")
     """
     del ttl_hash  # only used for functools.cache
+    if date and (start_date or end_date):
+        raise ValueError(
+            f"Cannot filter by specific date and date range at the same time: {date=}, {start_date=}, {end_date=}"
+        )
 
     capsule_or_pipeline_id = get_normalized_uuid(capsule_or_pipeline_id)
 
@@ -470,6 +602,23 @@ def search_computations(
     )
     if name is not None:
         records = [record for record in records if record["name"] == name]
+    if date is not None:
+        records = [
+            record
+            for record in records
+            if datetime.datetime.fromtimestamp(record["created"]).date()
+            == npc_session.DateRecord(date)
+        ]
+    elif start_date is not None or end_date is not None:
+        start_date = start_date or datetime.datetime.min
+        end_date = end_date or datetime.datetime.max
+        records = [
+            record
+            for record in records
+            if npc_session.DatetimeRecord(start_date).dt
+            <= datetime.datetime.fromtimestamp(record["created"])
+            <= npc_session.DatetimeRecord(end_date).dt
+        ]
     if attached_data_asset_id is not None:
         records = [
             record
@@ -527,7 +676,8 @@ def get_subject_data_assets(
     """
     Get all assets associated with a subject ID.
 
-    - uses the `subject` field in asset metadata
+    - uses the `subject id` field in asset metadata
+    - also includes any assets found with `subject_id` in title
     - `subject_id` will be cast to a string for searching
     - subject ID is not required to be a labtracks MID
     - assets are sorted by ascending creation date
@@ -555,26 +705,61 @@ def get_subject_data_assets(
     Additional search parameters can be supplied as kwargs:
     >>> filtered_assets = get_subject_data_assets(668759, type='dataset')
     """
-    del ttl_hash  # only used for functools.cache
-
     if "query" in search_params:
         raise ValueError(
             "Cannot provide 'query' as a search parameter: a new query will be created using 'subject id' field to search for assets"
         )
     search_params["query"] = get_data_asset_search_query(subject_id=subject_id)
-    search_params["sort_field"] = codeocean.data_asset.DataAssetSortBy.Created
-    search_params["sort_order"] = codeocean.components.SortOrder.Ascending
-    assets = search_data_assets(search_params)
-    if not assets and not npc_session.extract_subject(str(subject_id)):
+    t0 = time.time()
+    # get assets from CodeOcean:
+    from_co = search_data_assets(search_params) + search_data_assets(
+        {"query": str(subject_id)}
+    )
+    co_asset_ids = {asset.id for asset in from_co}
+    # get assets from DocDB:
+    try:
+        docdb_asset_ids = (
+            aind_session.utils.docdb_utils.get_codeocean_data_asset_ids_from_docdb(
+                subject_id=subject_id,
+                ttl_hash=ttl_hash,
+            )
+        )
+    except requests.exceptions.RequestException as exc:
+        logger.warning(
+            f"Failed to get data assets for {subject_id=} from DocDB: {exc=!r}"
+        )
+        docdb_asset_ids = []
+    from_docdb = []
+    if docdb_asset_ids:
+        for id_ in docdb_asset_ids:
+            if id_ in co_asset_ids:
+                continue
+            try:
+                asset = get_data_asset_model(id_)
+            except (requests.HTTPError, CodeOceanError) as exc:
+                status_code = _get_status(exc)
+                if status_code in (401, 404):
+                    logger.warning(
+                        f"Not authorized to access data asset ID obtained from DocDB: {subject_id=}, {id_=}"
+                    )
+                    continue
+                raise
+            else:
+                from_docdb.append(asset)
+    assets = from_co + tuple(from_docdb)
+    logger.debug(
+        f"Got {len(assets)} data asset(s) for subject {subject_id!r} in {time.time() - t0:.3f}s"
+    )
+    if not assets and npc_session.extract_subject(str(subject_id)) is None:
         logger.warning(
             f"No assets were found for {subject_id=}, which does not appear to be a Labtracks MID"
         )
-    return assets
+    return sort_by_created(assets)
 
 
 @functools.cache
-def get_session_data_assets(
-    session_id: str | npc_session.AINDSessionRecord,
+def get_data_assets(
+    name_startswith: str,
     ttl_hash: int | None = None,
     **search_params,
 ) -> tuple[codeocean.data_asset.DataAsset, ...]:
@@ -582,10 +767,6 @@ def get_session_data_assets(
     Get all data assets whose names start with the search term.
 
     - assets are sorted by ascending creation date
-    - searching with a partial session ID is not reliable:
-      - use `aind_session.get_sessions()` to search for sessions instead, which
-      can filter on subject ID, platform, and date range
-      - then examine the `assets` attribute on each returned object
     - provide additional search parameters to filter results, as schematized in `codeocean.data_asset.DataAssetSearchParams`:
     https://github.com/codeocean/codeocean-sdk-python/blob/4d9cf7342360820f3d9bd59470234be3e477883e/src/codeocean/data_asset.py#L199
 
@@ -596,7 +777,7 @@ def get_session_data_assets(
     Examples
     --------
     Use a full session ID:
-    >>> assets = get_session_data_assets('ecephys_676909_2023-12-13_13-43-40')
+    >>> assets = get_data_assets('ecephys_676909_2023-12-13_13-43-40')
     >>> type(assets[0])
     <class 'codeocean.data_asset.DataAsset'>
     >>> assets[0].created
@@ -607,25 +788,33 @@ def get_session_data_assets(
     ['ecephys', 'raw', '676909']
 
     Additional search parameters can be supplied as kwargs:
-    >>> filtered_assets = get_session_data_assets('ecephys_676909_2023-12-13_13-43-40', type='dataset')
+    >>> filtered_assets = get_data_assets('ecephys_676909_2023-12-13_13-43-40', type='dataset')
     >>> assert len(filtered_assets) > 0
+
+    >>> assert get_data_assets('SmartSPIM_738819_2024-06-21_13-48-58')
     """
     del ttl_hash  # only used for functools.cache
-    try:
-        session_id = npc_session.AINDSessionRecord(session_id)
-    except ValueError as exc:
-        raise ValueError(
-            "Querying the CodeOcean API with a partial session ID is not reliable: use `aind_session.get_sessions()` to search, then examine the `assets` attribute on each returned object"
-        ) from exc
     if "query" in search_params:
         raise ValueError(
             "Cannot provide 'query' as a search parameter: a new query will be created using the 'name' field to search for assets"
         )
-    search_params["query"] = get_data_asset_search_query(name=session_id)
+    search_params["query"] = get_data_asset_search_query(name=name_startswith[:20])
     search_params["sort_field"] = codeocean.data_asset.DataAssetSortBy.Created
     search_params["sort_order"] = codeocean.components.SortOrder.Ascending
-    assets = search_data_assets(search_params)
-    return assets
+
+    t0 = time.time()
+    search_results: tuple[dict[str, Any], ...] = search_data_assets(
+        search_params, as_dict=True
+    )
+    assets = [
+        codeocean.data_asset.DataAsset.from_dict(result)
+        for result in search_results
+        if str(result["name"]).startswith(name_startswith)
+    ]
+    logger.debug(
+        f"Got {len(assets)} data asset(s) matching {name_startswith!r} in {time.time() - t0:.3f}s"
+    )
+    return sort_by_created(assets)
 
 
 def is_raw_data_asset(
